(*
    SNU 4541.664A HW4 
*)
open Syn

structure Pretty =
struct

fun print_pgm pgm =
    let
        val indent = true
        val noindent = false
        val tab = ref 0
        fun print_indent() = 
            let
                fun print_tab(cnt) = if cnt > 0 then (print_string "  ";print_tab(cnt-1))
            in
                print_tab(!tab)
            end
        fun inc_tab() = tab := !tab + 1
        fun dec_tab() = tab := !tab - 1
        fun print_token tok indent =
            if indent then (print_indent(); print_string tok)
                      else print_string tok

        fun print_cmd cmd =
            case cmd of
                Skip => print_token "skip" indent 
            |   VarAssign(id,exp) => (print_token id indent;
                                      print_token " := " noindent;
                                      print_exp exp)
            |   DerefAssign(id,exp) => (print_token ("*"^id) indent;
                                        print_token " := " noindent;
                                        print_exp exp)
            |   Seq(cmd1, cmd2) => (print_cmd cmd1;
                                    print_string ";\n";
                                    print_cmd cmd2)
            |   If(bexp, cmd1, cmd2) => 
                    (print_token "if" indent;
                     print_bexp bexp; print_token " then\n" noindent;
                     inc_tab();
                     print_cmd cmd1;
                     dec_tab();
                     print_string "\n";print_token "else\n" indent;
                     inc_tab();
                     print_cmd cmd2;
                     dec_tab();
                     print_string "\n";print_token "end" indent)
            |   While(bexp, cmd) =>
                    (print_indent();print_string "while ";
                     print_bexp bexp;
                     print_token " do\n" noindent;
                     inc_tab(); 
                     print_cmd cmd;
                     dec_tab();
                     print_string "\n";print_token "end" indent)
        and print_bexp bexp = 
            case bexp of
                Blop(blop,exp1,exp2) =>
                    (print_string "(";
                     print_exp exp1;
                     print_blop blop;
                     print_exp exp2;
                     print_string ")")
            | Land(bexp1, bexp2) =>
                    (print_string "(";
                     print_bexp bexp1;
                     print_string " && ";
                     print_bexp bexp2;
                     print_string ")")

        and print_exp exp =
            case exp of
                ReadInt => print_string "readint"
            |   Num(n)  => print_int n
            |   Var(id) => print_string id
            |   Deref(id) => print_string ("(*"^id^")")
            |   AddrOf(id) => print_string ("(&"^id^")")
            |   Bnop(bnop,exp1, exp2) =>
                    (print_string "(";
                     print_exp exp1;
                     print_bnop bnop;
                     print_exp exp2;
                     print_string ")")
            |   Unop(unop,exp) =>
                    (print_string "(";
                     print_unop unop;
                     print_exp exp;
                     print_string ")")
        and print_bnop bop =
            case bop of
                Add => print_string "+"
        and print_blop blop =
            case blop of
                Eq  => print_string "="
            |   Lt  => print_string "<"
        and print_unop uop =
            case uop of
                UMinus => print_string "-"

    in
        print_cmd pgm; print_string "\n"
    end

end
